-- Deploy cosmos:cosmosschema to pg

BEGIN;

  CREATE SCHEMA cosmos

  CREATE TABLE cosmos.profile (
    id serial PRIMARY KEY,
    display_name text NOT NULL,
    email text NOT NULL,
    metadata JSONB NULL
  );
  CREATE INDEX profile_gin ON cosmos.profile USING gin (metadata);

  CREATE TABLE cosmos."user" (
    id serial PRIMARY KEY,
    username text NOT NULL,
    password_hash text,
    profile_id int NOT NULL REFERENCES cosmos.profile (id),
    metadata JSONB NULL
  );
  CREATE INDEX user_gin ON cosmos."user" USING gin (metadata);

  CREATE TABLE cosmos."role" (
    id serial PRIMARY KEY,
    name text NOT NULL,
    datatypekey text NULL,
    metadata JSONB NULL
  );

  CREATE TABLE cosmos.profile_role (
    id serial PRIMARY KEY,
    profile_id int NOT NULL REFERENCES cosmos.profile (id),
    role_id int NOT NULL REFERENCES cosmos."role" (id),
    datatypekeylist JSONB NOT NULL,
    metadata JSONB NULL
  );
  CREATE INDEX profile_role_gin ON cosmos.profile_role USING gin (metadata);

  CREATE TABLE cosmos.password_reset_token (
    id serial PRIMARY KEY,
    user_id int REFERENCES cosmos."user" (id),
    token_hash text NOT NULL
  );

  CREATE TABLE cosmos.verification_token (
    id serial PRIMARY KEY,
    user_id int REFERENCES cosmos."user" (id),
    token_hash text NOT NULL
  );

  CREATE TABLE cosmos.data(
      id serial,
      datatypekey text NOT NULL,
      key text NOT NULL,
      data_item JSONB NOT NULL,
      updated_on timestamp NOT NULL,
      PRIMARY KEY (id)
    );
  CREATE INDEX data_gin ON cosmos.data USING gin (data_item);
  CREATE UNIQUE INDEX data_key ON cosmos.data (dataTypeKey, key);

  CREATE TABLE cosmos.query(
      id serial,
      datatypekey text NOT NULL,
      key text NOT NULL,
      query JSONB NOT NULL,
      queryparams JSONB NULL,
      updated_on timestamp NOT NULL,
      PRIMARY KEY (id)
    );
  CREATE INDEX query_gin ON cosmos.query USING gin (query);
  CREATE UNIQUE INDEX query_key ON cosmos.query (datatypekey, key);

  CREATE TABLE cosmos.routes(
      id serial,
      parent_id int NULL,
      position int NOT NULL,
      components JSONB NOT NULL,
      route JSONB NOT NULL,
      updated_on timestamp NOT NULL,
      PRIMARY KEY (id)
    );
  ALTER TABLE cosmos.routes ADD FOREIGN KEY (parent_id) REFERENCES cosmos.routes (id);

  CREATE TABLE cosmos.routes_snapshot(
      id serial,
      key text NOT NULL,
      snapshot JSONB NOT NULL,
      created_on timestamp NOT NULL,
      PRIMARY KEY (id)
    );
  CREATE UNIQUE INDEX routes_snapshot_key ON cosmos.routes_snapshot (key);

  CREATE TABLE cosmos.site_release(
      id serial,
      snapshot_key text NULL,
      PRIMARY KEY (id)
    );

  CREATE TABLE cosmos.sessions
  (
    sid character varying(255) NOT NULL,
    sess json NOT NULL,
    expired timestamp with time zone NOT NULL,
    CONSTRAINT sessions_pkey PRIMARY KEY (sid)
  );

  INSERT INTO cosmos.site_release(id) VALUES (1);
COMMIT;
