import { KnexConnection } from "../data/knexConnection";
import { IdentityType } from "../core/data";
import { IQueryData } from "../core/query";
import { IProfile, INewUserRequest, IRole, IRoleKey, IUserProfile, IUserConfig } from "../core/user";
import { Notifications } from "../notification/notifications";
import { IRequest, IGetRequest } from "./expressRequest";
import { IUserCreationService, IUserVerificationService, IForgotPasswordService, IResetPasswordService, IChangePasswordService, IVerifyUserRequest, IForgotPasswordRequest, IResetPasswordRequest, ILoginRequest, ITokenRepository } from "rokot-auth";
import { IWebConfig } from "../core/routes";
import { ICosmosRoleRepository, ICosmosUserRepository, ICosmosProfileRepository } from "../data/repo/core";
export interface IChangePassword {
    oldPassword: string;
    newPassword: string;
}
export declare class AuthenticationController {
    private connection;
    private userCreationService;
    private userVerificationService;
    private forgotPasswordService;
    private resetPasswordService;
    private changePasswordService;
    private profileRepository;
    private userRepository;
    private roleRepository;
    private tokenRepository;
    private notifications;
    private webConfig;
    private userConfig;
    constructor(connection: KnexConnection, userCreationService: IUserCreationService, userVerificationService: IUserVerificationService, forgotPasswordService: IForgotPasswordService, resetPasswordService: IResetPasswordService, changePasswordService: IChangePasswordService, profileRepository: ICosmosProfileRepository, userRepository: ICosmosUserRepository, roleRepository: ICosmosRoleRepository, tokenRepository: ITokenRepository, notifications: Notifications, webConfig: IWebConfig, userConfig: IUserConfig);
    createUser(req: IRequest<INewUserRequest, void, void, void>): Promise<void>;
    createUserReminder(req: IRequest<{
        username: string;
    }, void, void, void>): Promise<void>;
    rejectUser(req: IRequest<INewUserRequest, void, void, void>): Promise<void>;
    activateUsers(req: IGetRequest<void, {
        count: string;
    }, void>): Promise<void>;
    updateUser(req: IRequest<IProfile, IProfile, void, void>): Promise<void>;
    updateProfile(req: IRequest<IProfile, IProfile, void, void>): Promise<void>;
    verifyUser(req: IRequest<IVerifyUserRequest, void, void, void>): Promise<void>;
    forgotPassword(req: IRequest<IForgotPasswordRequest, void, void, void>): Promise<void>;
    resetPassword(req: IRequest<IResetPasswordRequest, void, void, void>): Promise<void>;
    changePassword(req: IRequest<IChangePassword, void, void, void>): Promise<void>;
    login(req: IRequest<ILoginRequest, void, void, void>): Promise<void>;
    logout(req: IGetRequest<void, void, void>): Promise<void>;
    profile(req: IGetRequest<IProfile, void, void>): Promise<void>;
    adminProfile(req: IGetRequest<IProfile, void, void>): Promise<void>;
    roles(req: IRequest<IQueryData, IRole[], void, void>): Promise<void>;
    deleteUser(req: IGetRequest<void, {
        id: IdentityType;
    }, void>): Promise<void>;
    userProfiles(req: IRequest<IQueryData, IUserProfile[], IRoleKey, void>): Promise<void>;
    userProfilesCsv(req: IRequest<IQueryData, string, IRoleKey, void>): Promise<void>;
}
