"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const rokot_apicontroller_1 = require("rokot-apicontroller");
const injectTypes_1 = require("../ioc/injectTypes");
const manager_1 = require("../data/manager");
const query_1 = require("../core/query");
//import {RoleRestrictions} from "../users/roleRestriction";
const common_1 = require("./common");
// export async function findQuery(dataManager: DataManager, dataTypeKey: string, key: string) {
//   const query = await dataManager.query.executeQuery(createKeyQuery(key, dataTypeKey))
//   return query && query.length === 1 ? query[0] : undefined
// }
function validateQuery(query) {
    throw new Error("Invalid Query Body");
}
function qsArray(value) {
    return value.split(",");
}
function execute(promise, defaultValue) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (!promise) {
            return defaultValue;
        }
        return yield promise;
    });
}
function executeOrTrue(promise) {
    return execute(promise, "ok");
}
let DataController = class DataController {
    constructor(dataManager, dataInterceptionConfig) {
        this.dataManager = dataManager;
        this.dataInterceptionConfig = dataInterceptionConfig;
    }
    get(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const q = query_1.createKeyQuery(req.params.key, req.params.dataTypeKey);
            if (req.body) {
                q.references = req.body;
            }
            const config = this.dataInterceptionConfig && this.dataInterceptionConfig() || {};
            const canExec = yield executeOrTrue(config.canRunQuery && config.canRunQuery(q, req.user));
            switch (canExec) {
                case "fail":
                    req.sendForbidden();
                    break;
                case "failSilent":
                    req.sendOk();
                    break;
                default:
                    const useSnapshot = yield this.dataManager.useSnapshot(req);
                    const data = yield this.dataManager.data.getAll(useSnapshot, req.user, q);
                    req.sendOk(data[0]);
                    break;
            }
        });
    }
    delete(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const config = this.dataInterceptionConfig && this.dataInterceptionConfig() || {};
            const canExec = yield executeOrTrue(config.canDelete && config.canDelete(req.params, req.user));
            switch (canExec) {
                case "fail":
                    req.sendForbidden();
                    break;
                case "failSilent":
                    req.sendNoContent();
                    break;
                default:
                    yield this.dataManager.data.delete(req.params);
                    config.onDelete && config.onDelete(req.params);
                    req.sendNoContent();
                    break;
            }
        });
    }
    runQuery(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const config = this.dataInterceptionConfig && this.dataInterceptionConfig() || {};
            const canExec = yield executeOrTrue(config.canRunQuery && config.canRunQuery(req.body, req.user));
            switch (canExec) {
                case "fail":
                    req.sendForbidden();
                    break;
                case "failSilent":
                    req.sendOk([]);
                    break;
                default:
                    const useRelease = yield this.dataManager.useSnapshot(req);
                    const data = yield this.dataManager.data.getAll(useRelease, req.user, req.body, common_1.makePaging(req.query));
                    req.sendOk(data);
                    break;
            }
        });
    }
    // KGW: @api.middleware("protect")
    saveData(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const isUpdate = !!req.body.id;
                if (this.dataInterceptionConfig) {
                    const config = this.dataInterceptionConfig() || {};
                    if (isUpdate) {
                        const canExec = yield executeOrTrue(config.canUpdate && config.canUpdate(req.body, req.params.dataTypeKey, req.user));
                        switch (canExec) {
                            case "fail":
                                req.sendForbidden();
                                return;
                            case "failSilent":
                                req.sendCreated(req.body);
                                return;
                            default:
                                break;
                        }
                    }
                    else {
                        const canExec = yield executeOrTrue(config.canCreate && config.canCreate(req.body, req.params.dataTypeKey, req.user));
                        switch (canExec) {
                            case "fail":
                                req.sendForbidden();
                                return;
                            case "failSilent":
                                req.sendCreated(req.body);
                                return;
                            default:
                                break;
                        }
                    }
                }
                let data;
                try {
                    data = yield this.dataManager.data.save({ dataTypeKey: req.params.dataTypeKey }, req.body);
                }
                catch (e) {
                    if (e.message === "conflict") {
                        req.sendConflict();
                        return;
                    }
                    throw e;
                }
                if (req.query.refs) {
                    const q = query_1.createKeyQuery(data.key, req.params.dataTypeKey);
                    q.references = qsArray(req.query.refs);
                    const data1 = yield this.dataManager.data.getAll(false, req.user, q);
                    data = data1 && data1[0];
                    if (data) {
                        notifyOnSave(data, req.params.dataTypeKey, isUpdate, this.dataInterceptionConfig);
                        req.sendCreated(data);
                        return;
                    }
                }
                notifyOnSave(data, req.params.dataTypeKey, isUpdate, this.dataInterceptionConfig);
                req.sendCreated(data);
            }
            catch (e) {
                console.log(e.message || e);
                if (e.message && e.message.indexOf(`duplicate key value violates unique constraint "data_key"`)) {
                    req.sendValidation([{ attribute: "key", message: "Duplicate Key" }]);
                    return;
                }
                throw e;
            }
        });
    }
};
tslib_1.__decorate([
    rokot_apicontroller_1.api.route(":dataTypeKey/:key"),
    rokot_apicontroller_1.api.verbs("post"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], DataController.prototype, "get", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route(":dataTypeKey/:key"),
    rokot_apicontroller_1.api.verbs("delete"),
    rokot_apicontroller_1.api.middleware("adminprotect"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], DataController.prototype, "delete", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route(),
    rokot_apicontroller_1.api.verbs("post")
    //@api.bodyValidator(validateQuery)
    ,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], DataController.prototype, "runQuery", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route(":dataTypeKey"),
    rokot_apicontroller_1.api.verbs("post"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], DataController.prototype, "saveData", null);
DataController = tslib_1.__decorate([
    rokot_apicontroller_1.api.controller(injectTypes_1.InjectTypes.DataController, "/data"),
    tslib_1.__metadata("design:paramtypes", [manager_1.DataManager, Function])
], DataController);
exports.DataController = DataController;
function notifyOnSave(data, dataTypeKey, isUpdate, dataInterceptionConfig) {
    if (dataInterceptionConfig) {
        const config = dataInterceptionConfig() || {};
        if (isUpdate) {
            config.onUpdate && config.onUpdate(data, dataTypeKey);
        }
        else {
            config.onCreate && config.onCreate(data, dataTypeKey);
        }
    }
}
// @api.controller(InjectTypes.SeedDataController, "/seed")
// export class SeedDataController {
//   constructor(private dataManager: DataManager) { }
//
//   @api.route()
//   @api.verbs("get")
//   async getSeedData(req: IGetRequest<ISeedData, void, void>) {
//     const queries = await this.dataManager.query.getNamedQueries()
//     if (!queries || !queries.length) {
//       req.sendNoContent()
//       return;
//     }
//
//     const data: ISeedData = { queries: {}, dataTypes: {} };
//
//     const seeded = queries.filter(q => !q.params)
//     for (let i = 0; i < seeded.length; i++) {
//       const seed = seeded[i];
//       const foundQuery = await findQuery(this.dataManager, seed.dataTypeKey, seed.key)
//       if (!foundQuery) {
//         continue;
//       }
//       const foundData = await this.dataManager.data.executeQuery(foundQuery);
//       foundData.forEach(d => {
//         let dt = data.dataTypes[foundQuery.dataTypeKey];
//         if (!dt) {
//           dt = data.dataTypes[foundQuery.dataTypeKey] = {}
//         }
//         dt[d.id] = d;
//       })
//       let ddd = data.queries[foundQuery.dataTypeKey];
//       if (!ddd) {
//         ddd = data.queries[foundQuery.dataTypeKey] = []
//       }
//       ddd.push({ key: seed.key, refs: foundData.map(d => d.id) })
//     }
//
//     req.sendOk(data)
//   }
// }
