"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const _ = require("underscore");
function streamToBuffer(stream, encoding) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const array = yield streamToArray(stream, d => _.isString(d) ? new Buffer(d, encoding) : d);
        return Buffer.concat(array);
    });
}
exports.streamToBuffer = streamToBuffer;
function streamToString(stream, encoding) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const array = yield streamToArray(stream, d => _.isString(d) ? d : d.toString(encoding));
        return array.join("");
    });
}
exports.streamToString = streamToString;
function streamToArray(stream, convert) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        let arr = [];
        if (!stream) {
            return arr;
        }
        return yield new Promise((resolve, reject) => {
            stream.on('data', onData);
            stream.on('end', onEnd);
            stream.on('error', onEnd);
            stream.on('close', onClose);
            function onData(doc) {
                arr.push(convert(doc));
            }
            function onEnd(err) {
                if (err)
                    reject(err);
                else
                    resolve(arr);
                cleanup();
            }
            function onClose() {
                resolve(arr);
                cleanup();
            }
            function cleanup() {
                arr = null;
                stream.removeListener('data', onData);
                stream.removeListener('end', onEnd);
                stream.removeListener('error', onEnd);
                stream.removeListener('close', onClose);
            }
        });
    });
}
