"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const rokot_apicontroller_1 = require("rokot-apicontroller");
const bodyParser = require("body-parser");
const injectTypes_1 = require("../ioc/injectTypes");
const manager_1 = require("../data/manager");
const sharp = require("sharp");
const rokot_upload_backend_1 = require("rokot-upload-backend");
const streams_1 = require("./streams");
function fromMetadata(uploadMetadata, key, ticket, userToken) {
    return {
        key,
        ticket,
        fileName: uploadMetadata.name,
        fileSize: uploadMetadata.size,
        contentType: uploadMetadata.contentType,
        additionalMetadata: uploadMetadata.additionalMetadata,
        userToken,
        active: true
    };
}
exports.UploadFileKey = "upload-file";
/**
 * LocalToS3
 */
class LocalToS3 {
    constructor(s3Config, dataManager, localStore, page) {
        this.dataManager = dataManager;
        this.localStore = localStore;
        this.page = page;
        this.s3store = new rokot_upload_backend_1.S3FileStore(s3Config);
    }
    migrate(profile) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const data = yield this.dataManager.data.getAll(false, profile, { type: "query", dataTypeKey: exports.UploadFileKey, page: this.page });
            for (var index = 0; index < data.length; index++) {
                let upload = data[index];
                //if (upload && upload.additionalMetadata && upload.additionalMetadata.destroyed_at){
                // if (!upload || !upload.additionalMetadata || !upload.additionalMetadata.destroyed_at){
                //   console.log("IGNORED NOT DELETED", upload.id, upload.key)        
                //   continue
                // }
                try {
                    const md = yield this.s3store.getFileAdditionalMetadata(upload.key);
                    console.log("exists: skipped");
                    continue;
                }
                catch (e) {
                }
                let u = { name: upload.fileName, contentType: upload.contentType, size: upload.fileSize, additionalMetadata: upload.additionalMetadata };
                let buf;
                try {
                    let read = yield this.localStore.readFile(upload.key);
                    buf = yield streams_1.streamToBuffer(read);
                    if (index === 0) {
                        console.log(JSON.stringify(u), upload.id, upload.key);
                    }
                }
                catch (e) {
                    console.log("error: skipped", e);
                    continue;
                }
                //read.
                try {
                    let ticket = yield this.s3store.createFile(upload.key, u);
                    yield this.s3store.writeChunk(ticket, buf, 1, { byteCount: upload.fileSize, startAt: 0, endAt: upload.fileSize - 1, isLastChunk: true, totalLength: upload.fileSize });
                    let result = yield this.s3store.finalize(ticket);
                    console.log(index + (this.page && this.page.skip ? parseInt(this.page.skip) : 0));
                }
                catch (e) {
                    console.log("ERROR", e.message || e, upload.id, upload.key);
                }
            }
        });
    }
}
class Middleware {
}
// @api.middlewareFunction("json")
// static json = bodyParser.json()
Middleware.raw = bodyParser.raw({ limit: '10mb' });
tslib_1.__decorate([
    rokot_apicontroller_1.api.middlewareFunction("raw"),
    tslib_1.__metadata("design:type", Object)
], Middleware, "raw", void 0);
let UploadController = UploadController_1 = class UploadController {
    constructor(dataManager, createFileService, putChunkService, fileStore) {
        this.dataManager = dataManager;
        this.createFileService = createFileService;
        this.putChunkService = putChunkService;
        this.fileStore = fileStore;
    }
    createFile(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const result = yield this.createFileService.createFile(req.body);
            const md = fromMetadata(req.body, result.filename, result.ticket, req.native.request.user);
            yield this.dataManager.data.save({ dataTypeKey: exports.UploadFileKey }, md);
            req.sendOk(result);
        });
    }
    putChunk(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const ticket = req.headers["content-ticket"];
            const partNumber = parseInt(req.headers["content-part-number"]);
            const rangeHeader = req.headers["content-range"];
            const result = yield this.putChunkService.putChunk(ticket, req.body, partNumber, rangeHeader);
            if (result.complete) {
                const extract = getExtract(result.additionalMetadata);
                if (extract) {
                    const file = yield this.fileStore.readFile(result.filename);
                    let s = sharp().extract(extract);
                    if (extract.maxWidth && extract.width > extract.maxWidth) {
                        const scale = extract.maxWidth / extract.width;
                        s = s.resize(Math.round(extract.width * scale), Math.round(extract.height * scale));
                    }
                    const extractPipe = file.pipe(s);
                    // extractPipe.on('error', err => {
                    //     req.send(404)
                    // })
                    const buff = yield streams_1.streamToBuffer(extractPipe);
                    yield this.fileStore.upload(buff, `crop_${result.filename}`, { name: result.filename, size: buff.length, additionalMetadata: result.additionalMetadata, contentType: "image/jpg" });
                }
            }
            req.sendNoContent();
        });
    }
    getUpload(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const data = yield this.dataManager.data.getAll(false, req.user, { type: "key", dataTypeKey: exports.UploadFileKey, key: req.params.id });
            if (data && data.length) {
                req.sendOk(data[0]);
                return;
            }
            req.sendNoContent();
        });
    }
    migrateToS3(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!UploadController_1.migrateS3Config) {
                req.send(404);
                return;
            }
            const l = new LocalToS3(UploadController_1.migrateS3Config, this.dataManager, this.fileStore, req.query);
            l.migrate(req.user);
            req.sendNoContent();
        });
    }
    getContent(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const data = yield this.dataManager.data.get(false, req.user, { dataTypeKey: exports.UploadFileKey, key: req.params.id });
            if (data) {
                const key = data.key;
                const am = data.additionalMetadata;
                const crop = am && am.cropWidth;
                const readable = yield this.fileStore.readFile(crop ? `crop_${key}` : key);
                readable.on('error', err => {
                    req.send(404);
                });
                if (req.query.width) {
                    const width = parseInt(req.query.width);
                    const height = req.query.height ? parseInt(req.query.height) : undefined;
                    const resizePipe = readable.pipe(sharp().resize(width, height));
                    resizePipe.on('error', err => {
                        req.send(404);
                    });
                    resizePipe.pipe(req.native.response);
                    return;
                }
                req.native.response.contentType(data.contentType);
                readable.pipe(req.native.response);
                return;
            }
            req.sendNoContent();
        });
    }
};
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("upload"),
    rokot_apicontroller_1.api.verbs("post")
    //@api.middleware("json")
    // @api.middleware("protect")
    // @api.middleware("authenticate")
    ,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], UploadController.prototype, "createFile", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("upload"),
    rokot_apicontroller_1.api.verbs("put"),
    rokot_apicontroller_1.api.middleware("raw"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], UploadController.prototype, "putChunk", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("upload/:id"),
    rokot_apicontroller_1.api.verbs("get")
    //@api.bodyValidator(validateQuery)
    ,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], UploadController.prototype, "getUpload", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("uploadMigrate/s3"),
    rokot_apicontroller_1.api.verbs("get"),
    rokot_apicontroller_1.api.middleware("protect")
    //@api.bodyValidator(validateQuery)
    ,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], UploadController.prototype, "migrateToS3", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("content/:id"),
    rokot_apicontroller_1.api.verbs("get")
    //@api.bodyValidator(validateQuery)
    ,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], UploadController.prototype, "getContent", null);
UploadController = UploadController_1 = tslib_1.__decorate([
    rokot_apicontroller_1.api.controller(injectTypes_1.InjectTypes.UploadController),
    tslib_1.__metadata("design:paramtypes", [manager_1.DataManager, Object, Object, Object])
], UploadController);
exports.UploadController = UploadController;
function getExtract(additionalMetadata) {
    const width = additionalMetadata && additionalMetadata.cropWidth;
    if (width) {
        return {
            left: Number(additionalMetadata.cropX),
            top: Number(additionalMetadata.cropY),
            width: Number(additionalMetadata.cropWidth),
            height: Number(additionalMetadata.cropHeight),
            maxWidth: additionalMetadata.maxWidth ? Number(additionalMetadata.maxWidth) : 0
        };
    }
}
var UploadController_1;
