import { IIdentity, IDataTypeKeyed } from "./data";
export declare type Query = IQuery | IKeyQuery;
export declare function isKeyQuery(query: Query): query is IKeyQuery;
export declare function isQuery(query: Query): query is IQuery;
export declare function createKeyQuery(key: string, dataTypeKey: string): IKeyQuery;
export declare function createQuery(dataTypeKey: string): IQuery;
export declare type QueryType = "key" | "query";
export interface IQueryBase extends IIdentity, IDataTypeKeyed {
    type: QueryType;
}
export interface IQuerySearch {
    value: string;
    fields: string[];
}
export interface IQueryData extends IQueryReference {
    fields?: string[];
    search?: IQuerySearch;
    distinct?: boolean;
    distinctKeysQuery?: IQuery;
    sort?: IQuerySort[];
    where?: IQueryWhere[];
    page?: IQueryPage;
    paramValues?: {
        [key: string]: string;
    };
}
export interface IQuery extends IQueryBase, IQueryData {
    type: "query";
}
export interface IKeyQueryData extends IQueryReference {
    key: string;
    fields?: string[];
}
export interface IQueryReference {
    references?: string[];
}
export interface IKeyQuery extends IQueryBase, IKeyQueryData {
    type: "key";
}
export interface IQueryParam {
    name: string;
    value?: string | string[];
}
export interface IQuerySort {
    field: string;
    descending?: boolean;
}
export declare type QueryOperation = "eq" | "neq" | "between" | "gt" | "lt" | "in" | "array-contains-any" | "array-contains-all";
export interface IQueryWhere {
    field: string;
    operation: QueryOperation;
    value?: string | string[];
    subQuery?: IQuery;
}
export declare function ensureArray(values: string | string[]): string[];
export interface IQueryPage {
    take: number;
    skip?: number;
}
