import { IQueryWhere, Query } from "./query";
import { IIdentity, IDataTypeKeyed, IdentityType, IDataItem, IKeyedAndDataTypeKeyed } from "./data";
export interface IDataInterceptionConfigFactory {
    (): IDataInterceptionConfig;
}
export declare type DataInterceptionResult = "ok" | "fail" | "failSilent";
export interface IDataInterceptionConfig {
    canRunQuery?(query: Query, profile: IProfile): Promise<DataInterceptionResult>;
    canDelete?(dataInfo: IKeyedAndDataTypeKeyed, profile: IProfile): Promise<DataInterceptionResult>;
    canCreate?(dataItem: IDataItem, dataTypeKey: string, profile: IProfile): Promise<DataInterceptionResult>;
    canUpdate?(dataItem: IDataItem, dataTypeKey: string, profile: IProfile): Promise<DataInterceptionResult>;
    onDelete?(dataInfo: IKeyedAndDataTypeKeyed): void;
    onCreate?(dataItem: IDataItem, dataTypeKey: string): void;
    onUpdate?(dataItem: IDataItem, dataTypeKey: string): void;
}
export interface IRoleQueryRestriction extends IDataTypeKeyed {
    where: IQueryWhere[];
}
export interface IDataTypeRelation {
    dataTypeKey: string;
    field: string;
}
export interface IDataTypeRelationRestriction extends IDataTypeRelation {
    roleId: IdentityType;
}
export interface IRoleKey {
    roleId: IdentityType;
    key?: string;
    type?: "access" | "explicit";
}
export interface IRole extends IIdentity {
    name: string;
    dataTypeKey: string;
    metadata: IDataTypeRelation[];
}
export interface IProfileRole extends IIdentity {
    roleId: IdentityType;
    dataTypeKeyList: string[];
}
export interface IProfile extends INewUserRequest {
    id: number;
    email: string;
}
export interface IUserProfile extends IProfile {
    isActive: boolean;
    isPendingVerify: boolean;
    isPendingPasswordReset: boolean;
    lastLoginAt: string;
}
export interface INewUserRequest {
    username: string;
    displayName: string;
    metadata?: {
        [key: string]: any;
    };
    roles?: IProfileRole[];
}
export interface IUserConfig {
    csvExportRow?: (u: IUserProfile) => any;
}
