"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const schemaBuilder_1 = require("./schemaBuilder");
class KnexTable {
    constructor(schema, connection) {
        this.schema = schema;
        this.connection = connection;
    }
    static getIdRowValue(table, key, value) {
        return table.schema.defs[key].toRowData(value);
    }
    static getIdRowValues(table, key, value) {
        return table.schema.defs[key].toRowData(value);
    }
    static makeQueryBuilderTx(connection, tableName, tx) {
        if (!tx) {
            return connection.knex(tableName);
        }
        return tx.from(tableName);
    }
    makeQueryBuilder(tx) {
        return KnexTable.makeQueryBuilderTx(this.connection, this.schema.qualifiedName, tx);
    }
    insert(update) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.schema.allKeys;
            let qb = this.applyWhere(this.makeQueryBuilder(update.tx), this.schema, update.where);
            const insertData = this.schema.mapper.toTableRow(this.connection.knex.raw, update.data);
            const returning = update.returning;
            qb = this.applyReturning(this.schema, r => r ? qb.insert(insertData, r) : qb.insert(insertData), returning, false);
            qb = this.applyFactory(qb, update.factory);
            return returning ? this.selectInsertReturning(qb) : this.selectInsert(qb);
        });
    }
    update(update) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let qb = this.applyWhere(this.makeQueryBuilder(update.tx), this.schema, update.where);
            const updateData = this.schema.mapper.toTableRow(this.connection.knex.raw, update.data);
            qb = this.applyReturning(this.schema, r => r ? qb.update(updateData, r) : qb.update(updateData), update.returning, false);
            qb = this.applyFactory(qb, update.factory);
            return this.selectManyItems(qb);
        });
    }
    delete(del) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let qb = this.applyWhere(this.makeQueryBuilder(del.tx), this.schema, del.where);
            qb = this.applyReturning(this.schema, r => r ? qb.delete(r) : qb.delete(), del.returning, false);
            return qb;
        });
    }
    selectInsert(qb) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const ss = qb && (yield qb);
            return ss ? ss.rowCount : 0;
        });
    }
    selectInsertReturning(qb) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const ss = qb && (yield qb);
            const s = ss && ss[0];
            return s ? this.schema.mapper.fromTableRow(s) : s;
        });
    }
    selectFirst(qb) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const s = qb && (yield qb.first());
            return s ? this.schema.mapper.fromTableRow(s) : s;
        });
    }
    selectManyItems(qb) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const selected = qb && (yield qb);
            return selected && selected.map && selected.map(s => {
                const r = this.schema.mapper.fromTableRow(s);
                return r;
            }) || selected;
        });
    }
    select(select) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.selectQueryBuilder(select, q => this.selectManyItems(q));
        });
    }
    selectOne(select) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.selectQueryBuilder(select, q => this.selectFirst(q));
        });
    }
    query(select) {
        let qb = this.applyWhere(this.makeQueryBuilder(select.tx), this.schema, select.where);
        qb = this.applyReturning(this.schema, r => r ? qb.select(r) : qb, select.fields, true);
        qb = this.applyOrder(qb, this.schema, select.order);
        qb = KnexTable.applyPaging(qb, select.paging);
        return this.applyFactory(qb, select.factory);
    }
    selectQueryBuilder(select, finalizer) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const qb = this.query(select);
            return finalizer ? finalizer(qb) : qb;
        });
    }
    countWhere(where, tx, factory) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.scalar(i => i.count && Number(i.count), q => q.count(), where, tx, factory);
        });
    }
    maxWhere(field, where, tx, factory) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.scalarField(field, "max", (q, c) => q.max(c), where, tx, factory);
        });
    }
    minWhere(field, where, tx, factory) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.scalarField(field, "min", (q, c) => q.min(c), where, tx, factory);
        });
    }
    averageWhere(field, where, tx, factory) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.scalarField(field, "avg", (q, c) => q.avg(c), where, tx, factory);
        });
    }
    scalar(getValue, factoryMain, where, tx, factory) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let qb = this.applyWhere(this.makeQueryBuilder(tx), this.schema, where);
            qb = this.applyFactory(qb, factory);
            const selected = yield factoryMain(qb).first();
            return selected && getValue(selected);
        });
    }
    scalarField(field, name, factoryMain, where, tx, factory) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.scalar(i => i[name], (q) => factoryMain(q, this.schema.mapper.toTableColumns(this.connection.knex.raw, [field])[0]), where, tx, factory);
        });
    }
    static applyPaging(builder, paging) {
        if (paging) {
            let skip = +paging.skip;
            if (isNaN(skip)) {
                skip = 0;
            }
            return builder.offset(Math.max(0, skip)).limit(paging.take);
        }
        return builder;
    }
    applyFactory(qb, qbFactory) {
        if (qbFactory) {
            qb = qbFactory(qb);
            if (qb === undefined) {
                return undefined;
            }
        }
        if (KnexTable.debug) {
            console.log(qb.toQuery());
        }
        return qb;
    }
    applyWhere(qb, schema, where) {
        if (where) {
            const db = schema.mapper.toTableRow(this.connection.knex.raw, where);
            if (db) {
                Object.keys(db).forEach(k => {
                    const value = db[k];
                    if (schemaBuilder_1.SchemaBuilder.isRaw(value)) {
                        console.log("RAW", k);
                        qb = qb.whereRaw(value);
                    }
                    else if (value === null) {
                        qb = qb.whereNull(k);
                    }
                    else if (Array.isArray(value)) {
                        qb = qb.whereIn(k, value);
                    }
                    else {
                        qb = qb.where(k, value);
                    }
                });
            }
        }
        return qb;
    }
    applyReturning(schema, updateData, returning, forceSelectAll) {
        const hasReturning = returning && !!returning.length;
        if (hasReturning) {
            const cols = schema.mapper.toTableColumns(this.connection.knex.raw, returning);
            return updateData(cols);
        }
        else if (forceSelectAll) {
            const cols = schema.mapper.toTableColumns(this.connection.knex.raw, schema.allKeys);
            return updateData(cols);
        }
        else {
            schema.allKeys;
            return updateData();
        }
    }
    applyOrder(qb, schema, order) {
        if (order) {
            const keys = Object.keys(order);
            const cols = schema.mapper.toTableColumns(this.connection.knex.raw, keys);
            cols.forEach((col, idx) => {
                qb = qb.orderBy(col, order[keys[idx]] ? "DESC" : "ASC");
            });
        }
        return qb;
    }
}
KnexTable.debug = false;
exports.KnexTable = KnexTable;
