"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const query_1 = require("../../core/query");
const _ = require("underscore");
class SqlServerQueryBuilder {
    constructor(connection) {
        this.connection = connection;
    }
    applyQueryData(data, qb) {
        return qb;
    }
    processSort(jsonField, builder, sort) {
        return builder.orderByRaw(`${jsonField}->>'${sort.field}'${sort.descending ? " DESC" : ""}`);
    }
    processSearch(jsonField, builder, search) {
        return builder.where(jsonField, "like", `%${search}%`);
    }
    processWhere(jsonField, builder, where) {
        switch (where.operation) {
            case "in":
                return builder.whereRaw(`JSON_VALUE(${jsonField}, '$.${where.field}') IN (${_.range(0, query_1.ensureArray(where.value).length).map(i => "?").join(",")})`, where.value);
            case "neq":
                return builder.whereRaw(`JSON_VALUE(${jsonField}, '$.${where.field}') != ?`, where.value);
            case "eq":
                return builder.whereRaw(`JSON_VALUE(${jsonField}, '$.${where.field}') = ?`, where.value);
            case "between":
                return builder.whereRaw(`JSON_VALUE(${jsonField}, '$.${where.field}') = BETWEEN ? AND ?`, where.value);
            case "gt":
                return builder.whereRaw(`JSON_VALUE(${jsonField}, '$.${where.field}') > ?`, where.value);
            case "lt":
                return builder.whereRaw(`JSON_VALUE(${jsonField}, '$.${where.field}') < ?`, where.value);
        }
        return builder;
    }
}
exports.SqlServerQueryBuilder = SqlServerQueryBuilder;
