/// <reference types="knex" />
import * as knexImport from "knex";
import { IUserRepository, IDataContext } from "rokot-auth";
import { IKeyedAndDataTypeKeyed, IDataItem, IdentityType } from "../../core/data";
import { IProfile, IRole, IDataTypeRelationRestriction, IProfileRole, IUserProfile, IRoleKey } from "../../core/user";
import { Query, IQueryPage, IQueryData } from "../../core/query";
import { RoutesGather, ISiteRoute, ISiteRouteData, ISiteSnapshot } from "../../core/routes";
import { SnapshotKey } from "../../core/release";
export declare type DataParamType = {
    dataTypeKey: string;
};
export declare type QueryParamType = {
    key: string;
};
export interface ICosmosDataRepository {
    release(tx?: knexImport.Transaction): Promise<void>;
    releaseItem(dataTypeKey: string, dataKey: string, tx?: knexImport.Transaction): Promise<void>;
    delete(query: IKeyedAndDataTypeKeyed, tx?: knexImport.Transaction): Promise<void>;
    get<T extends IDataItem>(useSnapshot: boolean, currentUserProfile: IProfile, query: IKeyedAndDataTypeKeyed, tx?: knexImport.Transaction): Promise<T>;
    getAll<T extends IDataItem>(useSnapshot: boolean, currentUserProfile: IProfile, query: Query, page?: IQueryPage, tx?: knexImport.Transaction): Promise<T[]>;
    save(params: DataParamType, dataItem: IDataItem, tx?: knexImport.Transaction): Promise<IDataItem>;
    getPending(tx?: knexImport.Transaction): Promise<IKeyedAndDataTypeKeyed[]>;
}
export interface ICosmosRoutesRepository {
    addRoute(parentId: IdentityType, data: ISiteRouteData, tx?: knexImport.Transaction): Promise<IdentityType[]>;
    updateRoute(data: ISiteRoute, tx?: knexImport.Transaction): Promise<ISiteRoute>;
    deleteRoute(snapshotRepository: ICosmosSnapshotRepository, id: IdentityType, tx?: knexImport.Transaction): Promise<boolean>;
    makeRouteTree(snapshotRepository: ICosmosSnapshotRepository, mode: RoutesGather, id?: IdentityType, tx?: knexImport.Transaction): Promise<ISiteRoute>;
    gatherTree(snapshotRepository: ICosmosSnapshotRepository, mode: RoutesGather, id?: IdentityType, tx?: knexImport.Transaction): Promise<ISiteRoute>;
}
export interface ICosmosSnapshotRepository {
    hasSnapshot(tx?: knexImport.Transaction): Promise<boolean>;
    getSnapshotKey(): Promise<string>;
    getSnapshot(key: SnapshotKey, tx?: knexImport.Transaction): Promise<ISiteSnapshot>;
    saveSnapshot(tx?: knexImport.Transaction): Promise<{
        key: string;
        id?: IdentityType;
    }>;
}
export interface ICosmosProfileRepository {
    getProfileById(id: IdentityType): Promise<IProfile>;
    getProfiles(data?: IQueryData, roleKey?: IRoleKey): Promise<IUserProfile[]>;
    deleteProfile(profileId: IdentityType, context?: any): Promise<void>;
    createProfile(displayName: string, email: string, metadata: any, context?: IDataContext): Promise<IdentityType>;
    updateProfile(profile: IProfile, withRoles: boolean, context?: IDataContext): Promise<IProfile>;
}
export interface ICosmosRoleRepository {
    getRolesForDataType(dataTypeKey: string): Promise<IRole[]>;
    getRolesRelationsForDataType(dataTypeKey: string): Promise<IDataTypeRelationRestriction[]>;
    getRoles(data?: IQueryData): Promise<IRole[]>;
    deleteUserRole(profileId: IdentityType, context?: any): Promise<void>;
    updateUserRole(profileId: IdentityType, roles: IProfileRole[], context?: any): Promise<IProfileRole[]>;
    getRoleById(id: IdentityType): Promise<IRole>;
    getProfileRoleDictionary(profileIds: IdentityType[]): Promise<{
        [profileId: string]: IProfileRole[];
    }>;
    getProfileRoles(profileId: IdentityType): Promise<IProfileRole[]>;
}
export interface ICosmosUserRepository extends IUserRepository {
    getProfileUsername(profileId: IdentityType, context: IDataContext): Promise<string>;
    deleteUser(profileId: IdentityType, context?: IDataContext): Promise<boolean>;
    onUserLogin(username: string, context: IDataContext): Promise<void>;
}
export declare namespace Dates {
    function now(): string;
    function toIso(date: Date): string;
    function fromIso(date: string): Date;
}
