/// <reference types="knex" />
import * as knexImport from "knex";
import { Query, IQueryPage } from "../../core/query";
import { IDbQueryBuilder } from "../knexTable";
import { IDataItem, IKeyedAndDataTypeKeyed } from "../../core/data";
import { IProfile } from "../../core/user";
import { ICosmosDataRepository, ICosmosRoleRepository, DataParamType } from "./core";
import { IReleaseConfig } from "../../core/release";
import { Schema } from "./schema";
export declare function modifyUserQuery(currentUserProfile: IProfile, query: Query, roleRepository: ICosmosRoleRepository): Promise<Query>;
export declare class DataRepository implements ICosmosDataRepository {
    private schemas;
    private queryBuilder;
    private roleRepository;
    private releaseConfig;
    constructor(schemas: Schema.KnexTables, queryBuilder: IDbQueryBuilder, roleRepository: ICosmosRoleRepository, releaseConfig: IReleaseConfig);
    private getReleaseQuery1(tx?, factory?);
    release(tx?: knexImport.Transaction): Promise<void>;
    releaseItem(dataTypeKey: string, dataKey: string, tx?: knexImport.Transaction): Promise<void>;
    delete(query: IKeyedAndDataTypeKeyed, tx?: knexImport.Transaction): Promise<void>;
    getPending(tx?: knexImport.Transaction): Promise<IKeyedAndDataTypeKeyed[]>;
    get<T extends IDataItem>(useSnapshot: boolean, currentUserProfile: IProfile, query: IKeyedAndDataTypeKeyed, tx?: knexImport.Transaction): Promise<T>;
    getAll<T extends IDataItem>(useSnapshot: boolean, currentUserProfile: IProfile, query: Query, page?: IQueryPage, tx?: knexImport.Transaction): Promise<T[]>;
    private getAllData<T>(useSnapshot, currentUserProfile, query, page?, tx?);
    private dataQueryFactory<T>(builder, jsonDataField, useSnapshotForQuery, useSnapshot, currentUserProfile, query, page?);
    private preProcessQueryWhere(query, useSnapshot, currentUserProfile);
    private prepareDataItem(doc, d, dataTypeKey);
    private makeDataItem<T>(jsonDataField, query, d);
    private loadDataItemReferences<T>(useSnapshot, currentUserProfile, references, docs);
    private fetchDataItemReferences(useSnapshot, currentUserProfile, dataTypeKey, keys, docs, propName, refs);
    save(params: DataParamType, dataItem: IDataItem, tx?: knexImport.Transaction): Promise<IDataItem>;
    private insert(params, dataItem, tx?);
    private update(params, dataItem, tx?);
    private getUpsertData(jsonDataField, dataItem, params);
}
