/// <reference types="knex" />
import * as knexImport from "knex";
import { IComponent } from "../../core/components";
import { IdentityType } from "../../core/data";
import { ISiteRouteData, ISiteRoute, RoutesGather } from "../../core/routes";
import { ICosmosRoutesRepository, ICosmosSnapshotRepository } from "./core";
import { Schema } from "./schema";
export interface IRouteExtended {
    route: ISiteRouteData;
    parentId?: IdentityType;
    position?: number;
}
export declare class RoutesRepository implements ICosmosRoutesRepository {
    private schemas;
    constructor(schemas: Schema.KnexTables);
    makeRouteTree(snapshotRepository: ICosmosSnapshotRepository, mode: RoutesGather, id?: IdentityType, tx?: knexImport.Transaction): Promise<ISiteRoute>;
    moveRoute(data: ISiteRoute, direction: ""): Promise<void>;
    private moveUp(parentId, id, tx?);
    updateRoute(data: ISiteRoute, tx?: knexImport.Transaction): Promise<ISiteRoute>;
    deleteRoute(snapshotRepository: ICosmosSnapshotRepository, id: IdentityType, tx?: knexImport.Transaction): Promise<boolean>;
    private deleteRouteInternal(route, tx);
    private deleteById(id, tx);
    addRoute(parentId: IdentityType, data: ISiteRouteData, tx?: knexImport.Transaction): Promise<IdentityType[]>;
    private addRouteInternal(parentId, data, tx);
    insertDirect(params: IRouteExtended, components: IComponent[], returnColumns: (keyof Schema.Routes)[], tx?: knexImport.Transaction): Promise<any>;
    gatherTree(snapshotRepository: ICosmosSnapshotRepository, mode: RoutesGather, id?: IdentityType, tx?: knexImport.Transaction): Promise<ISiteRoute>;
    private gatherTreeTx(snapshotRepository, mode, tx, id?);
    protected getUpdateData(components: IComponent[], params: IRouteExtended): Schema.Routes;
}
