"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const consts_1 = require("../../core/consts");
const _ = require("underscore");
const tree_1 = require("../route/tree");
const core_1 = require("./core");
class RoutesRepository {
    constructor(schemas) {
        this.schemas = schemas;
    }
    makeRouteTree(snapshotRepository, mode, id, tx) {
        return this.gatherTree(snapshotRepository, mode, id, tx);
    }
    moveRoute(data, direction) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.schemas.connection.tx((tx) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const tree = yield this.moveUp(data.parentId, data.id, tx);
                //return this.deleteRouteInternal(tree, tx)
            }));
        });
    }
    moveUp(parentId, id, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const updateDatas = yield this.schemas.routes.select({ tx, factory: q => q.where(this.schemas.routes.schema.columns.parentId, parentId), order: { position: false } });
            const move = _.findIndex(updateDatas, d => d.id === id);
            if (move > 1) {
                const b = updateDatas[move];
                updateDatas[move] = updateDatas[move - 1];
                updateDatas[move - 1] = b;
                for (let i = 0; i < updateDatas.length; i++) {
                    yield this.schemas.routes.update({
                        data: {
                            position: i + 1
                        },
                        tx,
                        returning: ["id", "position"],
                        where: { id: updateDatas[i].id }
                    });
                }
                return true;
            }
            return false;
        });
    }
    updateRoute(data, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const up = yield this.schemas.routes.update({
                data: this.getUpdateData(data.components, {
                    route: tree_1.narrowSiteRoute(data)
                }),
                tx,
                returning: ["components", "id", "route", "parentId"],
                where: { id: data.id }
            })
                .then((docs) => docs[0]);
            if (!up) {
                return;
            }
            return {
                id: up.id,
                parentId: up.parentId,
                title: up.route.title,
                meta: up.route.meta,
                url: up.route.url,
                excludeFromNav: !!up.route.excludeFromNav,
                additionalClassName: up.route.additionalClassName,
                components: up.components,
            };
        });
    }
    deleteRoute(snapshotRepository, id, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const tree = yield this.gatherTree(snapshotRepository, "AllRoutes", id);
            if (tx) {
                return this.deleteRouteInternal(tree, tx);
            }
            return this.schemas.connection.tx(tx => {
                return this.deleteRouteInternal(tree, tx);
            });
        });
    }
    deleteRouteInternal(route, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (route.childRoutes && route.childRoutes.length) {
                yield Promise.all(route.childRoutes.map(r => this.deleteRouteInternal(r, tx)));
            }
            return this.deleteById(route.id, tx);
        });
    }
    deleteById(id, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this.schemas.routes.delete({ where: { id }, tx })
                .then(() => {
                return true;
            });
        });
    }
    addRoute(parentId, data, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (tx) {
                return this.addRouteInternal(parentId, data, tx);
            }
            return this.schemas.connection.tx((tx) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                return this.addRouteInternal(parentId, data, tx);
            }));
        });
    }
    addRouteInternal(parentId, data, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            //const qb = parentId ? q1.where(Schema.routesTable.fields.parentId, parentId) : q1.whereNull(Schema.routesTable.fields.parentId)
            let max = yield this.schemas.routes.maxWhere("position", parentId ? { parentId } : { parentId: null }, tx);
            const d = yield this.insertDirect({
                route: data,
                parentId: parentId || null,
                position: max + 1
            }, [consts_1.createEmptyComponent()], ["route", "position", "parentId"], tx);
            return [d.id];
        });
    }
    insertDirect(params, components, returnColumns, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this.schemas.routes.insert({ data: this.getUpdateData(components, params), tx, returning: ["id", "components", ...returnColumns] });
        });
    }
    gatherTree(snapshotRepository, mode, id, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (tx) {
                return this.gatherTreeTx(snapshotRepository, mode, tx, id);
            }
            return this.schemas.connection.tx(tx1 => {
                return this.gatherTreeTx(snapshotRepository, mode, tx1, id);
            });
        });
    }
    gatherTreeTx(snapshotRepository, mode, tx, id) {
        return tree_1.buildTree(snapshotRepository, this.schemas.routes, mode, tx, id).then(d => tree_1.convertRoute(d));
    }
    getUpdateData(components, params) {
        return {
            id: undefined,
            parentId: params.parentId,
            position: params.position,
            components: components,
            route: params.route,
            updatedOn: core_1.Dates.now()
        };
    }
}
exports.RoutesRepository = RoutesRepository;
