import { IdentityType } from "../../core/data";
import { IComponent } from "../../core/components";
import { ISiteRouteData, ISiteRoute } from "../../core/routes";
import { IDataTypeRelation } from "../../core/user";
import { SchemaBuilder } from "../schemaBuilder";
import { KnexTableSchemas } from '../schemas';
export declare namespace Schema {
    class Data {
        dataItem: {};
        snapshotDataItem: {};
        snapshotUpdatedOn: {};
        id: IdentityType;
        key: string;
        dataTypeKey: string;
        updatedOn: string;
        deletedOn: string;
    }
    class Role {
        id: IdentityType;
        name: string;
        dataTypeKey: string;
        metadata: IDataTypeRelation[];
    }
    class ProfileRole {
        id: IdentityType;
        profileId: IdentityType;
        roleId: IdentityType;
        dataTypeKeyList: string[];
        metadata: string;
    }
    class RoutesSnapshot {
        id: IdentityType;
        key: string;
        snapshot: ISiteRoute;
        createdOn: string;
    }
    class PasswordResetToken {
        id: IdentityType;
        userId: IdentityType;
        tokenHash: string;
    }
    class VerificationToken {
        id: IdentityType;
        userId: IdentityType;
        tokenHash: string;
    }
    class Profile {
        id: IdentityType;
        displayName: string;
        email: string;
        metadata: {};
    }
    class User {
        id: IdentityType;
        username: string;
        passwordHash: string;
        profileId: IdentityType;
        metadata: {
            created_at: string;
            last_login_at?: string;
        };
    }
    class Routes {
        id: IdentityType;
        parentId: IdentityType;
        position: number;
        components: IComponent[];
        route: ISiteRouteData;
        updatedOn: string;
    }
    const tableSchemas: SchemaBuilder.CreatedTableSchemas<{
        data: Data;
        passwordResetToken: PasswordResetToken;
        profile: Profile;
        profileRole: ProfileRole;
        role: Role;
        routes: Routes;
        routesSnapshot: RoutesSnapshot;
        user: User;
        verificationToken: VerificationToken;
    }>;
    type KnexTables = KnexTableSchemas<typeof tableSchemas._typeRef, typeof tableSchemas>;
}
