"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("./core");
function latestSnapshot(qb, schemas) {
    return qb.orderBy(schemas.routesSnapshot.schema.columns.createdOn, "DESC");
}
class SnapshotRepository {
    constructor(schemas, routes, webConfig) {
        this.schemas = schemas;
        this.routes = routes;
        this.webConfig = webConfig;
    }
    saveSnapshot(tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const tree = yield this.routes.gatherTree(this, "All");
            if (tree._isReleased) {
                return { key: tree._latestUpdatedOn };
            }
            let qb = this.schemas.routesSnapshot.insert({
                data: {
                    key: tree._latestUpdatedOn,
                    snapshot: tree,
                    createdOn: core_1.Dates.now()
                },
                tx,
                returning: ["id"]
            });
            return yield qb
                .then(i => ({ key: tree._latestUpdatedOn, id: i.id }));
        });
    }
    keyQuery(tx) {
        return this.schemas.routesSnapshot.select({ fields: ["key"], tx });
    }
    getSnapshotKey(tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this.schemas.routesSnapshot.selectOne({ fields: ["key"], tx, factory: q => latestSnapshot(q, this.schemas) }).then(first => {
                return first && first.key;
            });
            //return await latestSnapshot(this.keyQuery(tx)).then(i => i && i.key as string)
        });
    }
    getSnapshotKeys(tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this.keyQuery(tx).then(i => i && i.length && i.map(x => x.key));
        });
    }
    hasSnapshot(tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this.getSnapshotKey(tx).then(k => !!k);
        });
    }
    getSnapshot(key, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (key === "latest") {
                return yield this.fetchLatest(tx);
            }
            return this.fetchSnapshot(tx);
        });
    }
    fetchLatest(tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const tree = yield this.routes.gatherTree(this, "All", undefined, tx);
            if (this.webConfig.virtualRootPath) {
                tree.url = this.webConfig.virtualRootPath;
            }
            return { route: tree };
        });
    }
    fetchSnapshot(tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this.schemas.routesSnapshot.selectOne({ fields: ["id", "key", "snapshot"], tx, factory: q => latestSnapshot(q, this.schemas) }).then(first => {
                if (!first) {
                    return this.fetchLatest(tx);
                }
                if (this.webConfig.virtualRootPath) {
                    first.snapshot.url = this.webConfig.virtualRootPath;
                }
                return { key: first.key, id: first.id, route: first.snapshot };
            });
        });
    }
}
exports.SnapshotRepository = SnapshotRepository;
