"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
class TokenRepository {
    constructor(schemas) {
        this.schemas = schemas;
    }
    getVerificationTokenHashes(username, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const rows = yield this.schemas.connection.knex(this.schemas.verificationToken.schema.qualifiedName)
                .innerJoin(this.schemas.user.schema.qualifiedName, this.schemas.user.schema.qualifiedColumns.id, this.schemas.verificationToken.schema.qualifiedColumns.userId)
                .where(this.schemas.user.schema.qualifiedColumns.username, username)
                .select(this.schemas.verificationToken.schema.qualifiedColumns.tokenHash)
                .transacting(context);
            return rows.map(row => {
                const hashString = row[this.schemas.verificationToken.schema.columns.tokenHash];
                const parts = hashString.split("~");
                return {
                    version: +parts[0],
                    hash: parts[1]
                };
            });
        });
    }
    createVerificationTokenHash(username, verificationTokenHash, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const hashString = `${verificationTokenHash.version}~${verificationTokenHash.hash}`;
            const idQuery = yield this.schemas.user.selectOne({ fields: ["id"], where: { username }, tx: context });
            yield this.schemas.verificationToken.insert({
                data: {
                    userId: idQuery.id,
                    tokenHash: hashString
                },
                tx: context
            });
        });
    }
    destroyAllVerificationTokenHashes(username, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const idQuery = yield this.schemas.user.selectOne({ fields: ["id"], where: { username }, tx: context });
            yield this.schemas.verificationToken.delete({ where: { userId: idQuery.id }, tx: context });
        });
    }
    getPasswordResetTokenHashes(username, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const rows = yield this.schemas.connection.knex(this.schemas.passwordResetToken.schema.qualifiedName)
                .innerJoin(this.schemas.user.schema.qualifiedName, this.schemas.user.schema.qualifiedColumns.id, this.schemas.passwordResetToken.schema.qualifiedColumns.userId)
                .where(this.schemas.user.schema.qualifiedColumns.username, username)
                .select(this.schemas.passwordResetToken.schema.qualifiedColumns.tokenHash)
                .transacting(context);
            return rows.map(row => {
                const hashString = row[this.schemas.passwordResetToken.schema.columns.tokenHash];
                const parts = hashString.split("~");
                return {
                    version: +parts[0],
                    hash: parts[1]
                };
            });
        });
    }
    createPasswordResetTokenHash(username, passwordResetTokenHash, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const hashString = `${passwordResetTokenHash.version}~${passwordResetTokenHash.hash}`;
            const idQuery = yield this.schemas.user.selectOne({ fields: ["id"], where: { username }, tx: context });
            yield this.schemas.passwordResetToken.insert({ data: {
                    userId: idQuery.id,
                    tokenHash: hashString
                }, tx: context });
        });
    }
    destroyAllPasswordResetTokenHashes(username, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const idQuery = yield this.schemas.user.selectOne({ fields: ["id"], where: { username }, tx: context });
            yield this.schemas.passwordResetToken.delete({ where: { userId: idQuery.id }, tx: context });
        });
    }
}
exports.TokenRepository = TokenRepository;
