"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const rokot_auth_1 = require("rokot-auth");
const core_1 = require("./core");
function tx(context) {
    return context;
}
class UserRepository {
    constructor(schemas) {
        this.schemas = schemas;
    }
    getProfileUsername(profileId, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const rows = yield this.schemas.user.select({ fields: ["username"], where: { profileId }, tx: tx(context) });
            if (!rows.length) {
                return null;
            }
            const row = rows[0];
            return row.username;
        });
    }
    userExists(username, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const count = yield this.schemas.user.countWhere({ username: username.toLowerCase() }, tx(context));
            return count > 0;
        });
    }
    getUserByUsername(username, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const rows = yield this.schemas.user.selectOne({ fields: ["id", "username", "profileId"], where: { username: username.toLowerCase() }, tx: tx(context) });
            if (!rows) {
                return null;
            }
            return rows;
        });
    }
    deleteUser(profileId, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const rows = yield this.schemas.user.selectOne({ fields: ["id"], where: { profileId }, tx: context });
            const userId = rows && rows.id;
            if (!userId) {
                return false;
            }
            const d1 = yield this.schemas.verificationToken.delete({ where: { userId }, returning: ["id"], tx: context });
            const d2 = yield this.schemas.passwordResetToken.delete({ where: { userId }, tx: context });
            const d3 = yield this.schemas.user.delete({ where: { id: userId }, tx: context });
            return true;
        });
    }
    createUser(newUser, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.schemas.user.insert({
                data: {
                    username: newUser.username,
                    profileId: newUser["profileId"],
                    metadata: { created_at: core_1.Dates.now() }
                },
                tx: tx(context)
            });
        });
    }
    getPasswordHashByUsername(username, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const rows = yield this.schemas.user.selectOne({ fields: ["passwordHash"], where: { username: username.toLowerCase() }, tx: tx(context) });
            const hashString = rows && rows.passwordHash;
            if (!hashString) {
                return {
                    version: rokot_auth_1.versioningHasher.getLatestHasherVersion(),
                    hash: ""
                };
            }
            const parts = hashString.split("~");
            return {
                version: Number(parts[0]),
                hash: parts[1]
            };
        });
    }
    setUserPassword(username, passwordHash, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const hashString = `${passwordHash.version}~${passwordHash.hash}`;
            yield this.schemas.user.update({
                data: {
                    passwordHash: hashString
                },
                tx: tx(context),
                where: { username: username.toLowerCase() }
            });
        });
    }
    onUserLogin(username, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const lowerUsername = username.toLowerCase();
            const rows = yield this.schemas.user.selectOne({ fields: ["metadata"], where: { username: lowerUsername }, tx: tx(context) });
            let md = rows.metadata || { created_at: core_1.Dates.now() };
            md.last_login_at = core_1.Dates.now();
            yield this.schemas.user.update({
                data: {
                    metadata: md
                },
                tx: tx(context),
                where: { username: lowerUsername }
            });
        });
    }
}
exports.UserRepository = UserRepository;
