"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
// export interface IUpdateData extends IIdentity {
//   parent_id: IdentityType
//   position: number
//   route: ISiteRouteData
//   components: IComponent[]
// }
// export interface IRouteData extends IUpdateData, IIdentity {
//   key: string
//   childRoutes?: IRouteData[]
//   _latestUpdatedOn?: string
//   _isReleased?: boolean
// }
function buildTree(snapshotRepository, routesSchema, mode, tx, id) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const items = yield buildTreeRecursive(tx, routesSchema, mode, 0, undefined, id);
        const item = items && items[0];
        if (mode === "SingleRoute") {
            return item;
        }
        if (item) {
            const max = yield routesSchema.maxWhere("updatedOn", undefined, tx);
            item._latestUpdatedOn = max;
            const snapshotKey = yield snapshotRepository.getSnapshotKey();
            if (snapshotKey) {
                item._isReleased = snapshotKey == item._latestUpdatedOn;
            }
            else {
                item._isReleased = false;
            }
        }
        return item;
    });
}
exports.buildTree = buildTree;
function buildTreeRecursive(tx, routesSchema, mode, depth, parentId, id) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        let fields = ["id", "parentId", "position", "route"];
        if (mode === "All" || (mode === "SingleRoute" && depth === 0)) {
            fields.push("components");
        }
        let where;
        if (id && id !== "0" && id !== 0) {
            where = { id };
        }
        else if (parentId) {
            where = { parentId };
        }
        else {
            where = { parentId: null };
        }
        let items = yield routesSchema.select({ where, fields, order: { position: false }, tx });
        //const items = await qb
        if (items.length) {
            for (let i = 0; i < items.length; i++) {
                const current = items[i];
                current.key = `${current.id}`;
                if (mode !== "SingleRoute" || depth < 1) {
                    current.childRoutes = yield buildTreeRecursive(tx, routesSchema, mode, depth + 1, current.id);
                }
            }
        }
        return items;
    });
}
function convertRoute(data, isChild) {
    if (!data) {
        return;
    }
    const siteRoute = Object.assign({}, narrowSiteRoute(data.route), { id: data.id, parentId: data.parentId, components: data.components, childRoutes: data.childRoutes && data.childRoutes.map(i => convertRoute(i, true)) });
    if (!isChild) {
        siteRoute._latestUpdatedOn = data._latestUpdatedOn;
        siteRoute._isReleased = data._isReleased;
    }
    return siteRoute;
}
exports.convertRoute = convertRoute;
function narrowSiteRoute(data) {
    return {
        title: data.title,
        meta: data.meta,
        excludeFromNav: data.excludeFromNav,
        url: data.url,
        additionalClassName: data.additionalClassName,
        allowedRoles: data.allowedRoles
    };
}
exports.narrowSiteRoute = narrowSiteRoute;
