/// <reference types="knex" />
import * as knexImport from "knex";
export interface ITx {
}
export declare namespace SchemaBuilder {
    type PrimaryKeyKinds = "primary" | "primary-generated";
    type WhereValue<TValue> = TValue | TValue[] | knexImport.QueryBuilder | knexImport.Raw | knexImport.QueryBuilder[] | knexImport.Raw[];
    type WherePartial<T> = {
        [P in keyof T]?: WhereValue<T[P]>;
    };
    type TableSchemaCtor<T> = {
        [P in keyof T]: new () => T[P];
    };
    type TableSchemas<T> = {
        [P in keyof T]: SchemaBuilder.ISchema<T[P]>;
    };
    type CreatedTableSchemas<T> = TableSchemas<T> & {
        _typeRef: T;
    };
    interface IColumnMapper<TTable> {
        fromTableRow: (tableRow: any) => Partial<TTable>;
        toTableRow: (raw: knexImport.RawBuilder, data: WherePartial<TTable>) => any;
        toTableColumns: (raw: knexImport.RawBuilder, names: (keyof TTable)[]) => string[];
        toTableColumn: (name: keyof TTable) => string;
    }
    interface ISchema<TTable> {
        name: string;
        qualifiedName: string;
        allKeys: (keyof TTable)[];
        primaryKeyKind: PrimaryKeyKinds;
        primaryKeys: (keyof TTable)[];
        columns: Record<keyof TTable, string>;
        defs: Record<keyof TTable, IDbMap>;
        qualifiedColumns: Record<keyof TTable, string>;
        mapper: IColumnMapper<TTable>;
    }
    function makeColumn<TDb, TData>(col: IColumn<TDb, TData>): IColumn<TDb, TData>;
    function isRaw<T>(where: SchemaBuilder.WhereValue<T>): where is knexImport.Raw;
    function isQueryBuilder<T>(params: knexImport.QueryBuilder | T): params is knexImport.QueryBuilder;
    function createTableSchemas<T>(baseSchema: string, item: TableSchemaCtor<T>): CreatedTableSchemas<T>;
    interface IColumn<TDb, TData> {
        name?: string;
        /** Called during select: Modify the column name to apply some functional sql transform */
        sqlFromDb?(raw: knexImport.RawBuilder, name: string): knexImport.Raw;
        /**
         * Called during insert/update: Modify the column data to apply some functional sql transform
         * NOTE: convertToDb is not called when this method is used
         * */
        sqlToDb?(raw: knexImport.RawBuilder, name: string, data: TData): knexImport.Raw;
        /** Called during select: Modify the db data to the user data type */
        convertFromDb?(data: TDb): TData;
        /** Called during insert/update: Modify the user data to the db data type */
        convertToDb?(data: TData): TDb;
    }
    interface IDbMap {
        fromRow(row: any): any;
        toRowData(value: any): any;
    }
    function primaryKeyGenerated<TDb, TData>(column?: IColumn<TDb, TData> | string): PropertyDecorator;
    function primaryKey<TDb, TData>(column?: IColumn<TDb, TData> | string): PropertyDecorator;
    function column<TDb, TData>(column?: IColumn<TDb, TData> | string): PropertyDecorator;
    function table(name: string): ClassDecorator;
}
