export declare type IError = Error;
export declare module CustomErrors {
    class Error implements IError {
        name: string;
        message: string;
        stack: string;
        static captureStackTrace(object: any, objectConstructor?: any): any;
    }
    enum ErrorType {
        NotFoundError = 1,
        ValidationError = 2,
        NoInputError = 3,
        UnauthenticatedError = 4,
        UnauthorizedError = 5,
        InvalidLoginError = 6,
        ConflictError = 7,
    }
    abstract class CustomError extends Error {
        internalMessage: string;
        constructor(type: ErrorType, message: string, internalMessage?: string);
    }
    class NotFoundError extends CustomError {
        constructor(message: string);
    }
    class ValidationError extends CustomError {
        errors: IValidationError[];
        constructor(errors: IValidationError[]);
    }
    interface IValidationError {
        message: string;
        attribute: string;
        value: any;
    }
    class NoInputError extends CustomError {
        constructor();
    }
    class UnauthenticatedError extends CustomError {
        constructor(reason: string);
    }
    class UnauthorizedError extends CustomError {
        constructor(reason: string);
    }
    class ConflictError extends CustomError {
        constructor(reason: string);
    }
    class InvalidLoginError extends CustomError {
        constructor(reason: string);
    }
}
export default CustomErrors;
