"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var CustomErrors;
(function (CustomErrors) {
    let ErrorType;
    (function (ErrorType) {
        ErrorType[ErrorType["NotFoundError"] = 1] = "NotFoundError";
        ErrorType[ErrorType["ValidationError"] = 2] = "ValidationError";
        ErrorType[ErrorType["NoInputError"] = 3] = "NoInputError";
        ErrorType[ErrorType["UnauthenticatedError"] = 4] = "UnauthenticatedError";
        ErrorType[ErrorType["UnauthorizedError"] = 5] = "UnauthorizedError";
        ErrorType[ErrorType["InvalidLoginError"] = 6] = "InvalidLoginError";
        ErrorType[ErrorType["ConflictError"] = 7] = "ConflictError";
    })(ErrorType = CustomErrors.ErrorType || (CustomErrors.ErrorType = {}));
    class CustomError extends Error {
        constructor(type, message, internalMessage) {
            super();
            this.internalMessage = internalMessage;
            this.name = ErrorType[type];
            this.message = message;
        }
    }
    CustomErrors.CustomError = CustomError;
    class NotFoundError extends CustomError {
        constructor(message) {
            super(ErrorType.NotFoundError, message);
            Error.captureStackTrace(this, this.constructor);
        }
    }
    CustomErrors.NotFoundError = NotFoundError;
    class ValidationError extends CustomError {
        constructor(errors) {
            super(ErrorType.ValidationError, "Validation error");
            this.errors = errors;
            Error.captureStackTrace(this, this.constructor);
        }
    }
    CustomErrors.ValidationError = ValidationError;
    class NoInputError extends CustomError {
        constructor() {
            super(ErrorType.NoInputError, "No input provided");
            Error.captureStackTrace(this, this.constructor);
        }
    }
    CustomErrors.NoInputError = NoInputError;
    class UnauthenticatedError extends CustomError {
        constructor(reason) {
            super(ErrorType.UnauthenticatedError, "Unauthenticated", reason);
            Error.captureStackTrace(this, this.constructor);
        }
    }
    CustomErrors.UnauthenticatedError = UnauthenticatedError;
    class UnauthorizedError extends CustomError {
        constructor(reason) {
            super(ErrorType.UnauthorizedError, "Access denied", reason);
            Error.captureStackTrace(this, this.constructor);
        }
    }
    CustomErrors.UnauthorizedError = UnauthorizedError;
    class ConflictError extends CustomError {
        constructor(reason) {
            super(ErrorType.ConflictError, `Conflict: ${reason}`);
            Error.captureStackTrace(this, this.constructor);
        }
    }
    CustomErrors.ConflictError = ConflictError;
    class InvalidLoginError extends CustomError {
        constructor(reason) {
            super(ErrorType.InvalidLoginError, "Invalid Login", reason);
            Error.captureStackTrace(this, this.constructor);
        }
    }
    CustomErrors.InvalidLoginError = InvalidLoginError;
})(CustomErrors = exports.CustomErrors || (exports.CustomErrors = {}));
exports.default = CustomErrors;
