"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kernel_1 = require("./kernel");
const injectTypes_1 = require("./injectTypes");
const data_1 = require("../controllers/data");
// import { QueryController } from "../controllers/query";
const referenceData_1 = require("../controllers/referenceData");
const release_1 = require("../controllers/release");
const routeTree_1 = require("../controllers/routeTree");
const auth_1 = require("../controllers/auth");
const upload_1 = require("../controllers/upload");
const snapshot_1 = require("../controllers/snapshot");
const manager_1 = require("../data/manager");
const api_1 = require("../server/api");
const knexConnection_1 = require("../data/knexConnection");
const postGres_1 = require("../data/queryBuilders/postGres");
const sqlServer_1 = require("../data/queryBuilders/sqlServer");
const rokot_log_1 = require("rokot-log");
const rokot_notification_1 = require("rokot-notification");
const notifications_1 = require("../notification/notifications");
const dispatcher_1 = require("../notification/dispatcher");
const passport_1 = require("../server/passport");
const user_1 = require("../data/repo/user");
const role_1 = require("../data/repo/role");
const token_1 = require("../data/repo/token");
const profile_1 = require("../data/repo/profile");
const rokot_auth_1 = require("rokot-auth");
const rokot_upload_backend_1 = require("rokot-upload-backend");
const data_2 = require("../data/repo/data");
const routes_1 = require("../data/repo/routes");
const snapshot_2 = require("../data/repo/snapshot");
const unfurl_1 = require("../controllers/unfurl");
const test_1 = require("../controllers/test");
const schema_1 = require("../data/repo/schema");
const schemas_1 = require("../data/schemas");
class CreateFileService {
    constructor(fileStore, filenameGenerator) {
        this.fileStore = fileStore;
        this.filenameGenerator = filenameGenerator;
    }
    createFile(metadata) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const filename = `${this.filenameGenerator.generateFilename()}_${metadata.name}`;
            const ticket = yield this.fileStore.createFile(filename, metadata);
            return { filename, ticket };
        });
    }
}
exports.CreateFileService = CreateFileService;
//register(InjectTypes.SiteController, SiteController, InjectTypes.DataManager)
kernel_1.register(injectTypes_1.InjectTypes.ContentRangeParser, rokot_upload_backend_1.ContentRangeParser).inSingletonScope();
kernel_1.register(injectTypes_1.InjectTypes.FilenameGenerator, rokot_upload_backend_1.UuidFilenameGenerator).inSingletonScope();
kernel_1.register(injectTypes_1.InjectTypes.CreateFileService, CreateFileService, injectTypes_1.InjectTypes.FileStore, injectTypes_1.InjectTypes.FilenameGenerator).inSingletonScope();
kernel_1.register(injectTypes_1.InjectTypes.PutChunkService, rokot_upload_backend_1.PutChunkService, injectTypes_1.InjectTypes.FileStore, injectTypes_1.InjectTypes.ContentRangeParser).inSingletonScope();
kernel_1.register(injectTypes_1.InjectTypes.UploadController, upload_1.UploadController, injectTypes_1.InjectTypes.DataManager, injectTypes_1.InjectTypes.CreateFileService, injectTypes_1.InjectTypes.PutChunkService, injectTypes_1.InjectTypes.FileStore);
function bindFileStore(fileStoreConfig, logger) {
    if (rokot_upload_backend_1.isS3FileStoreConfig(fileStoreConfig)) {
        logger.info("--  S3 File System");
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.FileStoreConfig).toConstantValue(fileStoreConfig);
        kernel_1.register(injectTypes_1.InjectTypes.FileStore, rokot_upload_backend_1.S3FileStore, injectTypes_1.InjectTypes.FileStoreConfig).inSingletonScope();
        return;
    }
    if (rokot_upload_backend_1.isLocalFileStoreConfig(fileStoreConfig)) {
        logger.info("--  Local File System");
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.FileStoreConfig).toConstantValue(fileStoreConfig);
        kernel_1.register(injectTypes_1.InjectTypes.FileStore, rokot_upload_backend_1.LocalFileStore, injectTypes_1.InjectTypes.FileStoreConfig).inSingletonScope();
        return;
    }
    logger.error("--  No File System defined", JSON.stringify(fileStoreConfig));
}
kernel_1.register(injectTypes_1.InjectTypes.SnapshotController, snapshot_1.SnapshotController, injectTypes_1.InjectTypes.DataManager);
kernel_1.register(injectTypes_1.InjectTypes.RouteTreeController, routeTree_1.RouteTreeController, injectTypes_1.InjectTypes.DataManager);
kernel_1.register(injectTypes_1.InjectTypes.ReleaseController, release_1.ReleaseController, injectTypes_1.InjectTypes.DataManager);
kernel_1.register(injectTypes_1.InjectTypes.UnfurlController, unfurl_1.UnfurlController);
kernel_1.register(injectTypes_1.InjectTypes.TestController, test_1.TestController, injectTypes_1.InjectTypes.KnexTableSchemas);
kernel_1.register(injectTypes_1.InjectTypes.DataController, data_1.DataController, injectTypes_1.InjectTypes.DataManager, injectTypes_1.InjectTypes.DataInterceptionConfig);
// register(InjectTypes.QueryController, QueryController, InjectTypes.DataManager)
kernel_1.register(injectTypes_1.InjectTypes.ReferenceDataController, referenceData_1.ReferenceDataController, injectTypes_1.InjectTypes.DataManager);
kernel_1.register(injectTypes_1.InjectTypes.Api, api_1.Api, injectTypes_1.InjectTypes.ApiConfig, injectTypes_1.InjectTypes.PassportAuth);
kernel_1.register(injectTypes_1.InjectTypes.KnexConnection, knexConnection_1.KnexConnection, injectTypes_1.InjectTypes.KnexConfig).inSingletonScope();
kernel_1.kernel.bind(injectTypes_1.InjectTypes.KnexTableSchemas).toDynamicValue(c => schemas_1.getKnexTables(schema_1.Schema.tableSchemas, c.container.get(injectTypes_1.InjectTypes.KnexConnection))).inSingletonScope();
kernel_1.register(injectTypes_1.InjectTypes.DataManager, manager_1.DataManager, injectTypes_1.InjectTypes.KnexConnection, injectTypes_1.InjectTypes.DataRepository, injectTypes_1.InjectTypes.RoutesRepository, injectTypes_1.InjectTypes.SnapshotRepository, injectTypes_1.InjectTypes.ProfileRepository, injectTypes_1.InjectTypes.RoleRepository, injectTypes_1.InjectTypes.UserRepository); //.inSingletonScope()
kernel_1.register(injectTypes_1.InjectTypes.AuthenticationController, auth_1.AuthenticationController, injectTypes_1.InjectTypes.KnexConnection, injectTypes_1.InjectTypes.UserCreationService, injectTypes_1.InjectTypes.UserVerificationService, injectTypes_1.InjectTypes.ForgotPasswordService, injectTypes_1.InjectTypes.ResetPasswordService, injectTypes_1.InjectTypes.ChangePasswordService, injectTypes_1.InjectTypes.ProfileRepository, injectTypes_1.InjectTypes.UserRepository, injectTypes_1.InjectTypes.RoleRepository, injectTypes_1.InjectTypes.TokenRepository, injectTypes_1.InjectTypes.Notifications, injectTypes_1.InjectTypes.WebConfig, injectTypes_1.InjectTypes.UserConfig);
kernel_1.register(injectTypes_1.InjectTypes.RoutesRepository, routes_1.RoutesRepository, injectTypes_1.InjectTypes.KnexTableSchemas);
kernel_1.register(injectTypes_1.InjectTypes.SnapshotRepository, snapshot_2.SnapshotRepository, injectTypes_1.InjectTypes.KnexTableSchemas, injectTypes_1.InjectTypes.RoutesRepository, injectTypes_1.InjectTypes.WebConfig);
kernel_1.register(injectTypes_1.InjectTypes.RoleRepository, role_1.RoleRepository, injectTypes_1.InjectTypes.KnexTableSchemas, injectTypes_1.InjectTypes.QueryBuilder);
kernel_1.register(injectTypes_1.InjectTypes.UserRepository, user_1.UserRepository, injectTypes_1.InjectTypes.KnexTableSchemas);
kernel_1.register(injectTypes_1.InjectTypes.ProfileRepository, profile_1.ProfileRepository, injectTypes_1.InjectTypes.KnexTableSchemas, injectTypes_1.InjectTypes.QueryBuilder, injectTypes_1.InjectTypes.RoleRepository);
kernel_1.register(injectTypes_1.InjectTypes.DataRepository, data_2.DataRepository, injectTypes_1.InjectTypes.KnexTableSchemas, injectTypes_1.InjectTypes.QueryBuilder, injectTypes_1.InjectTypes.RoleRepository, injectTypes_1.InjectTypes.ReleaseConfig);
kernel_1.register(injectTypes_1.InjectTypes.TokenRepository, token_1.TokenRepository, injectTypes_1.InjectTypes.KnexTableSchemas);
kernel_1.register(injectTypes_1.InjectTypes.PassportAuth, passport_1.PassportAuth, injectTypes_1.InjectTypes.KnexConnection, injectTypes_1.InjectTypes.ProfileRepository, injectTypes_1.InjectTypes.LoginService);
function getQueryBuilder(client) {
    switch (client) {
        case "pg":
            return postGres_1.PostgresQueryBuilder;
        case "mssql":
            return sqlServer_1.SqlServerQueryBuilder;
        default:
            console.log(`Unable to start service - unknown knex client '${client}' - only 'pg' and 'mssql' supported!`);
            return;
    }
}
function createLogger(level = "info") {
    return rokot_log_1.ConsoleLogger.create("cosmos-api", { level });
}
function isLogger(logger) {
    return logger && !!logger.info;
}
function run(config, loggerInit) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const logger = isLogger(loggerInit) ? loggerInit : createLogger(loggerInit);
        const queryBuilder = getQueryBuilder(config.knex.client);
        if (!queryBuilder) {
            return false;
        }
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.TemplateProcessor).toDynamicValue(c => new rokot_notification_1.UnderscoreFileSystemNestedTemplateProcessor("./templates", logger)).inSingletonScope();
        bindFileStore(config.fileStore, logger);
        const dispatcher = yield dispatcher_1.createDispatcher(logger, config.sendgrid);
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.Logger).toConstantValue(logger);
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.Notifications).toConstantValue(new notifications_1.Notifications(dispatcher));
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.DataInterceptionConfig).toConstantValue(config.dataInterception);
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.WebConfig).toConstantValue(config.web);
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.ReleaseConfig).toConstantValue(config.release);
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.UserConfig).toConstantValue(config.user);
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.ApiConfig).toConstantValue(config.api);
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.KnexConfig).toConstantValue(config.knex);
        kernel_1.register(injectTypes_1.InjectTypes.QueryBuilder, queryBuilder, injectTypes_1.InjectTypes.KnexConnection);
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.UserCreationService).toDynamicValue((c) => rokot_auth_1.makeUserCreationService(c.container.get(injectTypes_1.InjectTypes.UserRepository), c.container.get(injectTypes_1.InjectTypes.TokenRepository), logger)).inSingletonScope();
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.UserVerificationService).toDynamicValue((c) => rokot_auth_1.makeUserVerificationService(c.container.get(injectTypes_1.InjectTypes.UserRepository), c.container.get(injectTypes_1.InjectTypes.TokenRepository), logger)).inSingletonScope();
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.ForgotPasswordService).toDynamicValue((c) => rokot_auth_1.makeForgotPasswordService(c.container.get(injectTypes_1.InjectTypes.UserRepository), c.container.get(injectTypes_1.InjectTypes.TokenRepository), logger)).inSingletonScope();
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.ResetPasswordService).toDynamicValue((c) => rokot_auth_1.makeResetPasswordService(c.container.get(injectTypes_1.InjectTypes.UserRepository), c.container.get(injectTypes_1.InjectTypes.TokenRepository), logger)).inSingletonScope();
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.ChangePasswordService).toDynamicValue((c) => rokot_auth_1.makeChangePasswordService(c.container.get(injectTypes_1.InjectTypes.UserRepository), c.container.get(injectTypes_1.InjectTypes.TokenRepository), logger)).inSingletonScope();
        kernel_1.kernel.bind(injectTypes_1.InjectTypes.LoginService).toDynamicValue((c) => rokot_auth_1.makeLoginService(c.container.get(injectTypes_1.InjectTypes.UserRepository), logger)).inSingletonScope();
        config.onBeforeRun && config.onBeforeRun(kernel_1.kernel);
        const connection = kernel_1.kernel.get(injectTypes_1.InjectTypes.KnexConnection);
        const ok = yield connection.test();
        if (!ok) {
            yield connection.destroy();
            return false;
        }
        const api = kernel_1.kernel.get(injectTypes_1.InjectTypes.Api);
        if (!api.run()) {
            yield connection.destroy();
            return false;
        }
        return true;
    });
}
exports.run = run;
