"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const rokot_apicontroller_1 = require("rokot-apicontroller");
const expressRequest_1 = require("../controllers/expressRequest");
const rokot_log_1 = require("rokot-log");
const cors = require("cors");
const kernel_1 = require("../ioc/kernel");
const errorRequestHandler = (err, req, res, next) => {
    console.log("Unexpected error:::", err, req.url, req.body);
    res.status(400).send(err.message || err);
};
class Api {
    constructor(config, passportAuth) {
        this.config = config;
        this.passportAuth = passportAuth;
    }
    run() {
        const app = express();
        app.set("etag", false);
        app.use(cors({ origin: true, credentials: true }));
        app.disable('x-powered-by');
        this.config.afterCreate && this.config.afterCreate(app);
        this.passportAuth.apply(app, this.config);
        this.config.afterAuthConfig && this.config.afterAuthConfig(app);
        const logger = rokot_log_1.ConsoleLogger.create("Api Routes", { level: "trace" });
        const runtimeApi = rokot_apicontroller_1.createDefaultRuntimeApi(logger);
        const builder = new expressRequest_1.CustomExpressRouteBuilder(logger, app, (cc, n) => kernel_1.kernel.get(n));
        const ok = builder.build(runtimeApi);
        if (!ok) {
            console.log("Unable to build express routes - Service stopping!");
            return false;
        }
        app.use(this.config.errorRequestHandler || errorRequestHandler);
        this.config.afterRouteConfig && this.config.afterRouteConfig(app);
        app.listen(this.config.port, () => {
            console.log(`Cosmos API listening on port ${this.config.port}!`);
        });
        return true;
    }
}
exports.Api = Api;
