"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const expressSession = require("express-session");
const bodyParser = require("body-parser");
const passport = require("passport");
const passport_local_1 = require("passport-local");
const expressSessionKnexFactory = require("connect-session-knex");
const KnexSessionStore = expressSessionKnexFactory(expressSession);
class PassportAuth {
    constructor(connection, profileRepository, loginService) {
        this.connection = connection;
        this.profileRepository = profileRepository;
        this.loginService = loginService;
    }
    apply(app, config) {
        passport.use(new passport_local_1.Strategy((username, password, done) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let user;
            try {
                user = yield this.loginService.login({ username, password });
            }
            catch (err) {
                return done(err);
            }
            if (!user) {
                return done(null, false);
            }
            done(null, user);
        })));
        passport.serializeUser((user, done) => {
            done(null, user.profileId);
        });
        passport.deserializeUser((id, done) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const profile = yield this.profileRepository.getProfileById(id);
                done(null, profile);
            }
            catch (err) {
                done(err, null);
            }
        }));
        const sessionConfig = {
            store: new KnexSessionStore({ knex: this.connection.knex, tablename: 'sessions' }),
            secret: config.authSecret || "t0p-Secr3T-se(r£t",
            resave: false,
            saveUninitialized: false
        };
        if (config.cookieDomain) {
            sessionConfig.cookie = { domain: config.cookieDomain };
        }
        app.use(expressSession(sessionConfig));
        app.use(passport.initialize());
        app.use(passport.session());
        app.use(bodyParser.json());
        app.use(bodyParser.urlencoded({ extended: true }));
    }
}
exports.PassportAuth = PassportAuth;
