"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rokot_test_1 = require("rokot-test");
const propertyPathResolver_1 = require("../users/propertyPathResolver");
const ppr = new propertyPathResolver_1.PropertyPathResolver();
function expectThrow(user, propertyName) {
    const assertion = rokot_test_1.expect(() => {
        ppr.getValue(user, propertyName);
        console.log("Expected Throw for property", propertyName);
    }).to.throw();
}
function expectOk(user, propertyName, expected) {
    rokot_test_1.expect(ppr.getValue(user, propertyName)).to.eq(expected);
}
function makeRole(id) {
    return { id, name: "test" };
}
function makeCountry(id) {
    return { id, name: "UK" };
}
function makeAddress(id) {
    return { id, first: "first", second: "second", postcode: "postcode", country: makeCountry(198) };
}
function makeUser() {
    return { id: 2, userName: "phil", division: 2, address: makeAddress(19), displayName: "Mow", emails: ["phil@rocketmakers.com"], roles: [makeRole(4), makeRole(9)] };
}
describe("Property Path Resolver", () => {
    it("should fail with empty or null property names", () => {
        const user = makeUser();
        expectThrow(user, "");
        expectThrow(user, undefined);
        expectThrow(user, null);
    });
    it("should find first level props", () => {
        const user = makeUser();
        expectOk(user, "id", user.id);
        expectOk(user, "userName", user.userName);
        expectOk(user, "division", user.division);
        expectOk(user, "address", user.address);
        expectOk(user, "displayName", user.displayName);
        expectOk(user, "emails", user.emails);
        expectOk(user, "roles", user.roles);
    });
    it("should not fail for first level properties not found - just return undefined (a valid jscript operation!)", () => {
        const user = makeUser();
        expectOk(user, "_id", undefined);
        expectOk(user, "notThere", undefined);
        expectOk(user, "defoNotThere", undefined);
    });
    it("should find second level props of first level property", () => {
        const user = makeUser();
        expectOk(user, "address.first", user.address.first);
        expectOk(user, "address.country", user.address.country);
        expectOk(user, "address.second", user.address.second);
        expectOk(user, "address.id", user.address.id);
    });
    it("should not fail for second level properties not found - just return undefined (a valid jscript operation!)", () => {
        const user = makeUser();
        expectOk(user, "address._first", undefined);
        expectOk(user, "address._second", undefined);
        expectOk(user, "address.ids", undefined);
    });
    it("should find items of first level array property", () => {
        const user = makeUser();
        [0, 1].forEach(i => {
            expectOk(user, `roles.${i}`, user.roles[i]);
            expectOk(user, `roles.${i}.id`, user.roles[i].id);
            expectOk(user, `roles.${i}.name`, user.roles[i].name);
        });
    });
    it("should not fail for 'any other name access' of first level array property - just return undefined (a valid jscript operation!)", () => {
        const user = makeUser();
        expectOk(user, "address._first", undefined);
        expectOk(user, "address._second", undefined);
        expectOk(user, "address.ids", undefined);
    });
    it("should fail when child property requested of first level property not found", () => {
        const user = makeUser();
        expectThrow(user, "address1.first");
        expectThrow(user, "_roles.first");
    });
});
