"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const query_1 = require("../core/query");
const _ = require("underscore");
class QueryParamResolver {
    static queryParamsUsedInValue(value) {
        if (!value) {
            return [];
        }
        if ((value.match(/\{/g) || []).length !== (value.match(/\}/g) || []).length) {
            //console.log("count { != count }")
            throw new Error("Invalid Query Parameter value, a field must contain an equal number of '{' and '}' characters");
        }
        if (value.indexOf("{{") > -1) {
            //console.log("{{")
            throw new Error("Invalid Query Parameter value, a field cannot contain '{{' characters");
        }
        if (value.indexOf("}}") > -1) {
            //console.log("}}")
            throw new Error("Invalid Query Parameter value, a field cannot contain '}}' characters");
        }
        if (value.indexOf("{}") > -1) {
            //console.log("{}")
            throw new Error("Invalid Query Parameter value, a field cannot contain an empty '{}' variable");
        }
        const m = value.match(this.regex);
        if (m && m.length) {
            return _.uniq(m).map(p => {
                const variableName = p.substr(1, p.length - 2);
                if (variableName.indexOf(" ") > -1) {
                    //console.log("space")
                    throw new Error("Invalid Query Parameter value, a variable cannot contain any space characters");
                }
                if (variableName.indexOf("{") > -1) {
                    //console.log("{")
                    throw new Error("Invalid Query Parameter value, a variable cannot contain a '{' characters");
                }
                if (variableName.indexOf("}") > -1) {
                    //console.log("}")
                    throw new Error("Invalid Query Parameter value, a variable cannot contain a '}' characters");
                }
                return variableName;
            });
        }
        return [];
    }
    static makeParameters(query) {
        const params = this.findParameters(query.where);
        return params && params.length && params.map(p => ({ name: p }));
    }
    static findQueryParameters(query) {
        return this.findParameters(query.where);
    }
    static findParameters(wheres) {
        if (!wheres || !wheres.length) {
            return;
        }
        return _.uniq(_.flatten(wheres.map(w => query_1.ensureArray(w.value).map(v => this.queryParamsUsedInValue(v)))));
    }
    static replaceParameters(wheres, paramObject) {
        wheres.forEach(w => {
            if (!w.value) {
                return;
            }
            if (_.isArray(w.value)) {
                w.value = w.value.map(v => {
                    const used = QueryParamResolver.queryParamsUsedInValue(v);
                    if (used) {
                        used.forEach(u => {
                            v = v.replace(`{${u}}`, paramObject[u]);
                        });
                    }
                    return v;
                });
                return;
            }
            let value = w.value;
            const used = QueryParamResolver.queryParamsUsedInValue(value);
            if (used && used.length) {
                if (used.length === 1 && _.isArray(paramObject[used[0]])) {
                    w.value = paramObject[used[0]];
                    return;
                }
                used.forEach(u => {
                    value = value.replace(`{${u}}`, paramObject[u]);
                });
                w.value = value;
            }
        });
    }
    static resolve(query, params) {
        const required = QueryParamResolver.findParameters(query.where);
        if (!required || !required.length) {
            return query;
        }
        if (!params || !params.length) {
            throw new Error(`Query Params are required: supply '${required.join(",")}'`);
        }
        const missing = _.difference(required, params.map(p => p.name));
        if (missing.length) {
            throw new Error(`Invalid Query Params: missing: '${missing.join(",")}' supplied: '${params.map(p => p.name).join(",")}'`);
        }
        const paramObject = _.object(params.map(p => [p.name, p.value]));
        QueryParamResolver.replaceParameters(query.where, paramObject);
        return query;
    }
}
QueryParamResolver.regex = /\{([^}]+)\}/g;
exports.QueryParamResolver = QueryParamResolver;
